@@ -7,7 +7,6 @@
 #' @import methods
 #' @import RCurl
 #' @importFrom graphics barplot lines
-#' @importFrom rjson fromJSON
 #' @importFrom stats binomial Gamma gaussian poisson runif quantile screeplot
 #' @importFrom statmod tweedie
 #' @importFrom tools md5sum
@@ -29,10 +28,16 @@
   stopifnot(is(conn, "H2OConnection"))
   stopifnot(is.character(urlSuffix))
 
+  if (conn@https) {
+    scheme = "https"
+  } else {
+    scheme = "http"
+  }
+
   if (missing(h2oRestApiVersion))
-    sprintf("http://%s:%s/%s", conn@ip, as.character(conn@port), urlSuffix)
+    sprintf("%s://%s:%s/%s", scheme, conn@ip, as.character(conn@port), urlSuffix)
   else
-    sprintf("http://%s:%s/%s/%s", conn@ip, as.character(conn@port), h2oRestApiVersion, urlSuffix)
+    sprintf("%s://%s:%s/%s/%s", scheme, conn@ip, as.character(conn@port), h2oRestApiVersion, urlSuffix)
 }
 
 .h2o.doRawREST <- function(conn = h2o.getConnection(), h2oRestApiVersion, urlSuffix, parms, method, fileUploadInfo, ...) {
@@ -59,6 +64,22 @@
 
   url = .h2o.calcBaseURL(conn = conn, h2oRestApiVersion = h2oRestApiVersion, urlSuffix = urlSuffix)
 
+  opts = curlOptions()
+  if (!is.na(conn@username)) {
+    if (is.na(conn@password)) {
+      stop("Password not specified")
+    }
+
+    userpwd = sprintf("%s:%s", conn@username, conn@password)
+    basicAuth = 1L
+    opts = curlOptions(userpwd = userpwd, httpauth = basicAuth, .opts = opts)
+  }
+  if (conn@https) {
+    if (conn@insecure) {
+      opts = curlOptions(ssl.verifypeer = 0L, .opts = opts)
+    }
+  }
+
   queryString = ""
   i = 1L
   while (i <= length(parms)) {
@@ -105,22 +126,27 @@
   beginTimeSeconds = as.numeric(proc.time())[3L]
 
   tmp <- NULL
-  if (method == "GET") {
-    h = basicHeaderGatherer()
-    tmp = tryCatch(getURL(url = url,
-                          headerfunction = h$update,
-                          useragent = R.version.string,
-                          timeout = timeout_secs),
-                   error = function(x) { .__curlError <<- TRUE; .__curlErrorMessage <<- x$message })
+  if ((method == "GET") || (method == "DELETE")) {
+    h <- basicHeaderGatherer()
+    t <- basicTextGatherer(.mapUnicode = FALSE)
+    tmp <- tryCatch(curlPerform(url = url,
+                                customrequest = method,
+                                writefunction = t$update,
+                                headerfunction = h$update,
+                                useragent=R.version.string,
+                                verbose = FALSE,
+                                timeout = timeout_secs,
+                                .opts = opts),
+                    error = function(x) { .__curlError <<- TRUE; .__curlErrorMessage <<- x$message })
     if (! .__curlError) {
       httpStatusCode = as.numeric(h$value()["status"])
       httpStatusMessage = h$value()["statusMessage"]
-      payload = tmp
+      payload = t$value()
     }
   } else if (! missing(fileUploadInfo)) {
     stopifnot(method == "POST")
     h = basicHeaderGatherer()
-    t = basicTextGatherer()
+    t = basicTextGatherer(.mapUnicode = FALSE)
     tmp = tryCatch(postForm(uri = url,
                             .params = list(fileUploadInfo = fileUploadInfo),
                             .opts=curlOptions(writefunction = t$update,
@@ -128,7 +154,8 @@
                                               useragent = R.version.string,
                                               httpheader = c('Expect' = ''),
                                               verbose = FALSE,
-                                              timeout = timeout_secs)),
+                                              timeout = timeout_secs,
+                                              .opts = opts)),
                    error = function(x) { .__curlError <<- TRUE; .__curlErrorMessage <<- x$message })
     if (! .__curlError) {
       httpStatusCode = as.numeric(h$value()["status"])
@@ -137,7 +164,7 @@
     }
   } else if (method == "POST") {
     h = basicHeaderGatherer()
-    t = basicTextGatherer()
+    t = basicTextGatherer(.mapUnicode = FALSE)
     tmp = tryCatch(curlPerform(url = url,
                                postfields = postBody,
                                writefunction = t$update,
@@ -145,31 +172,15 @@
                                useragent = R.version.string,
                                httpheader = c('Expect' = ''),
                                verbose = FALSE,
-                               timeout = timeout_secs),
+                               timeout = timeout_secs,
+                               .opts = opts),
                    error = function(x) { .__curlError <<- TRUE; .__curlErrorMessage <<- x$message })
     if (! .__curlError) {
       httpStatusCode = as.numeric(h$value()["status"])
       httpStatusMessage = h$value()["statusMessage"]
       payload = t$value()
     }
-  } else if (method == "DELETE") {
-    h <- basicHeaderGatherer()
-    t <- basicTextGatherer()
-    tmp <- tryCatch(curlPerform(url = url,
-                                customrequest = method,
-                                writefunction = t$update,
-                                headerfunction = h$update,
-                                useragent=R.version.string,
-                                verbose = FALSE,
-                                timeout = timeout_secs),
-                    error = function(x) { .__curlError <<- TRUE; .__curlErrorMessage <<- x$message })
-    if (! .__curlError) {
-      httpStatusCode = as.numeric(h$value()["status"])
-      httpStatusMessage = h$value()["statusMessage"]
-      payload = t$value()
-    }
-  }
-  else {
+  } else {
     message = sprintf("Unknown HTTP method %s", method)
     stop(message)
   }
@@ -303,7 +314,7 @@
     cat(sprintf("ERROR: Unexpected HTTP Status code: %d %s (url = %s)\n", rv$httpStatusCode, rv$httpStatusMessage, rv$url))
     cat("\n")
 
-    jsonObject = fromJSON(rv$payload)
+    jsonObject = jsonlite::fromJSON(rv$payload, simplifyDataFrame=FALSE)
 
     exceptionType = jsonObject$exception_type
     if (! is.null(exceptionType)) {
@@ -390,6 +401,7 @@
           tbl <- do.call(cbind, lapply(x$data, sapply, function(cell) if (is.null(cell)) "" else cell))
         cnms <- sapply(x$columns, `[[`, "name")
         fmts <- sapply(x$columns, `[[`, "format")
+        descr <- x$description
         if( x$name=="Confusion Matrix" ) {
           colnames(tbl) <- make.unique(cnms)
           rownames(tbl) <- make.unique(c(cnms[1:(length(cnms)-2)], "Totals"))
@@ -426,6 +438,7 @@
         if( x$name == "Confusion Matrix") attr(tbl, "header") <- paste0(x$name, " - (", x$description, ")")
         else                              attr(tbl, "header")  <- x$name
         attr(tbl, "formats") <- fmts
+        attr(tbl, "description")   <- descr
         oldClass(tbl) <- c("H2OTable", "data.frame")
         x <- tbl
       }
@@ -434,11 +447,7 @@
     }
     x
   }
-  # hack that counters the fact that RCurl will escape already escaped string
-  txt <- gsub("\\\"","\"",txt); 
-  txt <- gsub("\\\\,","\\,",txt);
-
-  res <- processMatrices(fromJSON(txt, ...))
+  res <- processMatrices(txt, ...)
   processTables(res)
 }
 
@@ -461,10 +470,11 @@ print.H2OTable <- function(x, header=TRUE, ...) {
   # format columns
   formats <- attr(x, "formats")
   xx <- x
-
-  for (j in seq_along(x)) {
-    if( formats[j] == "%d" ) formats[j] <- "%f"
-    xx[[j]] <- .format.helper(x[[j]], formats[j])
+  if( !is.null(formats) ) {  # might be NULL if resulted from slicing H2OTable (no need for full blown slice method on H2OTable... allow to be data frame at that point)
+    for (j in seq_along(x)) {
+#      if( formats[j] == "%d" ) formats[j] <- "%f"
+      xx[[j]] <- .format.helper(x[[j]], formats[j])
+    }
   }
   # drop empty columns
   nz <- unlist(lapply(xx, function(y) any(nzchar(y))), use.names = FALSE)
@@ -475,8 +485,13 @@ print.H2OTable <- function(x, header=TRUE, ...) {
 
   # use data.frame print method
   xx <- data.frame(xx, check.names = FALSE, stringsAsFactors = FALSE)
-  if( header && !is.null(attr(x, "header")) )
-    cat(attr(x, "header"), ":\n", sep = "")
+  if( header && !is.null(attr(x, "header")) ) {
+    cat(attr(x, "header"), ":", sep="")
+    if( !is.null(attr(x,"description")) )
+      cat(" ", attr(x, "description"),sep="")
+    cat("\n")
+  }
+
 
   # pretty print the frame if it is large (e.g. > 20 rows)
   nr <- nrow(xx)
@@ -520,7 +535,7 @@ print.H2OTable <- function(x, header=TRUE, ...) {
   else                    rawREST <- .h2o.doSafeREST(conn = conn, h2oRestApiVersion = h2oRestApiVersion, urlSuffix = page, parms = .params, method = method)
 
   if( raw ) rawREST
-  else      .h2o.fromJSON(rawREST)
+  else      .h2o.fromJSON(jsonlite::fromJSON(rawREST,simplifyDataFrame=FALSE))
 }
 
 
@@ -538,7 +553,23 @@ h2o.clusterIsUp <- function(conn = h2o.getConnection()) {
 
   rv <- .h2o.doRawGET(conn = conn, urlSuffix = "")
 
-  !rv$curlError && ((rv$httpStatusCode == 200) || (rv$httpStatusCode == 301))
+  if (rv$curlError) {
+    return(FALSE)
+  }
+
+  if (rv$httpStatusCode == 401) {
+    warning("401 Unauthorized Access.  Did you forget to provide a username and password?")
+  }
+
+  if (rv$httpStatusCode == 200) {
+    return(TRUE)
+  }
+
+  if (rv$httpStatusCode == 301) {
+    return(TRUE)
+  }
+
+  return(FALSE)
 }
 
 #'
@@ -564,7 +595,7 @@ h2o.clusterInfo <- function(conn = h2o.getConnection()) {
     stop(sprintf("Cannot connect to H2O instance at %s", h2o.getBaseURL(conn)))
   }
 
-  res <- .h2o.fromJSON(.h2o.doSafeGET(conn = conn, urlSuffix = .h2o.__CLOUD))
+  res <- .h2o.fromJSON(jsonlite::fromJSON(.h2o.doSafeGET(conn = conn, urlSuffix = .h2o.__CLOUD), simplifyDataFrame=FALSE))
   nodeInfo <- res$nodes
   numCPU <- sum(sapply(nodeInfo,function(x) as.numeric(x['num_cpus'])))
 
@@ -574,7 +605,7 @@ h2o.clusterInfo <- function(conn = h2o.getConnection()) {
     # to post its information yet.
     threeSeconds = 3L
     Sys.sleep(threeSeconds)
-    res <- .h2o.fromJSON(.h2o.doSafeGET(conn = conn, urlSuffix = .h2o.__CLOUD))
+    res <- .h2o.fromJSON(jsonlite::fromJSON(.h2o.doSafeGET(conn = conn, urlSuffix = .h2o.__CLOUD), simplifyDataFrame=FALSE))
   }
 
   nodeInfo <- res$nodes
@@ -583,7 +614,15 @@ h2o.clusterInfo <- function(conn = h2o.getConnection()) {
   allowedCPU = sum(sapply(nodeInfo,function(x) as.numeric(x['cpus_allowed'])))
   clusterHealth <- all(sapply(nodeInfo,function(x) as.logical(x['healthy'])))
 
-  cat("R is connected to H2O cluster:\n")
+  is_client <- res$is_client
+  if (is.null(is_client)) {
+    is_client <- FALSE
+  }
+  assign("IS_CLIENT", is_client, .pkg.env)
+  m <- ": \n"
+  if( is_client ) m <- " (in client mode): \n"
+
+  cat(paste0("R is connected to the H2O cluster", m))
   cat("    H2O cluster uptime:        ", .readableTime(as.numeric(res$cloud_uptime_millis)), "\n")
   cat("    H2O cluster version:       ", res$version, "\n")
   cat("    H2O cluster name:          ", res$cloud_name, "\n")
@@ -604,7 +643,6 @@ h2o.clusterInfo <- function(conn = h2o.getConnection()) {
 #' Warn if there are sick nodes.
 .h2o.__checkConnectionHealth <- function(conn = h2o.getConnection()) {
   rv <- .h2o.doGET(conn = conn, urlSuffix = .h2o.__CLOUD)
-
   if (rv$curlError) {
     ip = conn@ip
     port = conn@port
@@ -619,7 +657,7 @@ h2o.clusterInfo <- function(conn = h2o.getConnection()) {
          sprintf("H2O returned HTTP status %d (%s)", rv$httpStatusCode, rv$httpStatusMessage))
   }
 
-  cloudStatus <- .h2o.fromJSON(rv$payload)
+  cloudStatus <- .h2o.fromJSON(jsonlite::fromJSON(rv$payload, simplifyDataFrame=FALSE))
   nodes = cloudStatus$nodes
   overallHealthy = TRUE
   for (i in 1:length(nodes)) {
@@ -635,10 +673,13 @@ h2o.clusterInfo <- function(conn = h2o.getConnection()) {
     url <- .h2o.calcBaseURL(conn = conn, h2oRestApiVersion = .h2o.__REST_API_VERSION, urlSuffix = .h2o.__CLOUD)
     warning(paste0("Check H2O cluster status here: ", url, "\n", collapse = ""), immediate. = T)
   }
-
-  0L
 }
 
+#'
+#' Check Client Mode Connection
+#'
+#' @export
+h2o.is_client <- function() get("IS_CLIENT", .pkg.env)
 
 #-----------------------------------------------------------------------------------------------------------------------
 #   Job Polling
@@ -648,13 +689,12 @@ h2o.clusterInfo <- function(conn = h2o.getConnection()) {
   if (progressBar) {
     pb <- txtProgressBar(style = 3L)
   }
-
   keepRunning <- TRUE
   tryCatch({
     while (keepRunning) {
       myJobUrlSuffix <- paste0(.h2o.__JOBS, "/", job_key)
       rawResponse <- .h2o.doSafeGET(conn,urlSuffix = myJobUrlSuffix)
-      jsonObject <- .h2o.fromJSON(rawResponse)
+      jsonObject <- .h2o.fromJSON(jsonlite::fromJSON(rawResponse, simplifyDataFrame=FALSE))
       jobs <- jsonObject$jobs
       if (length(jobs) > 1) {
         stop("Job list has more than 1 entry")
@@ -665,6 +705,7 @@ h2o.clusterInfo <- function(conn = h2o.getConnection()) {
       job = jobs[[1]]
 
       status = job$status
+#      print(paste0("Job status: ", status))
       stopifnot(is.character(status))
 
       # check failed up front...
