@@ -29,6 +29,11 @@
 #'        is an initial center. For lists of vectors, each vector is an
 #'        initial center.
 #' @param seed (Optional) Random seed used to initialize the cluster centroids.
+#' @param nfolds (Optional) Number of folds for cross-validation. If \code{nfolds >= 2}, then \code{validation} must remain empty.
+#' @param fold_column (Optional) Column with cross-validation fold index assignment per observation
+#' @param fold_assignment Cross-validation fold assignment scheme, if fold_column is not specified
+#'        Must be "AUTO", "Random" or "Modulo"
+#' @param keep_cross_validation_predictions Whether to keep the predictions of the cross-validation models
 #' @return Returns an object of class \linkS4class{H2OClusteringModel}.
 #' @seealso \code{\link{h2o.cluster_sizes}}, \code{\link{h2o.totss}}, \code{\link{h2o.num_iterations}},
 #'          \code{\link{h2o.betweenss}}, \code{\link{h2o.tot_withinss}}, \code{\link{h2o.withinss}},
@@ -45,7 +50,11 @@ h2o.kmeans <- function(training_frame, x, k,
                        max_iterations = 1000,
                        standardize = TRUE,
                        init = c("Furthest","Random", "PlusPlus"),
-                       seed)
+                       seed,
+                       nfolds = 0,
+                       fold_column = NULL,
+                       fold_assignment = c("AUTO","Random","Modulo"),
+                       keep_cross_validation_predictions = FALSE)
 {
   # Training_frame may be a key or an H2OFrame object
   if (!inherits(training_frame, "H2OFrame"))
@@ -57,7 +66,7 @@ h2o.kmeans <- function(training_frame, x, k,
   # Gather user input
   parms <- list()
   if( !(missing(x)) )
-    parms$ignored_columns <- .verify_datacols(training_frame, x)$cols_ignore
+      parms$ignored_columns <- .verify_datacols(training_frame, x)$cols_ignore
   if(!missing(k))
     parms$k <- k
   parms$training_frame <- training_frame
@@ -71,6 +80,11 @@ h2o.kmeans <- function(training_frame, x, k,
     parms$init <- init
   if(!missing(seed))
     parms$seed <- seed
+  if (!missing(nfolds))
+    parms$nfolds <- nfolds
+  if( !missing(fold_column) )               parms$fold_column            <- fold_column
+  if( !missing(fold_assignment) )           parms$fold_assignment        <- fold_assignment
+  if( !missing(keep_cross_validation_predictions) )  parms$keep_cross_validation_predictions  <- keep_cross_validation_predictions
 
   # Check if init is an acceptable set of user-specified starting points
   if( is.data.frame(init) || is.matrix(init) || is.list(init) || inherits(init, "H2OFrame") ) {
